/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/core/utils/Logger", [
	"dojo/_base/declare",
	"dojo/_base/config",
	"dojo/json",
	"dojo/topic",
	"ibm/usmi/widgets/util/Constants"
], function (declare, config, json, topic, Constants) {
	"use strict";
	return declare(null, {
		className : null,
		traceName : "",
		level : 0, // error

		constructor : function (newLoggerName, classOrigin) {
			var traceConfigElement = null, loggerSpecificity = 0;
			this.traceName = newLoggerName;
			this.className = classOrigin || "(unknown class)";

			if (config.trace) {
				// "all" overrides individual class entries
				if (config.trace.all) {
					this.level = this._translateLevel("all", config.trace.all);
				}
				for (traceConfigElement in config.trace) {
					if (config.trace.hasOwnProperty(traceConfigElement)) {
						if (newLoggerName.indexOf(traceConfigElement) === 0) {
							// have a match.  want to make sure that a more specific rule
							// hasn't been defined for the logger.
							if (loggerSpecificity < traceConfigElement.split('.').length) {
								// more dots means more specific
								loggerSpecificity = traceConfigElement.split('.').length;
								this.level = this._translateLevel(name, config.trace[traceConfigElement]);
								if (newLoggerName === traceConfigElement) {
									// found an exact match for the logger.  no need to search further.
									break;
								}
							}
						}
					}
				}
			}
		},

		/////////////////////////////
		// public methods
		/////////////////////////////

		isError : function () {
			return (this.level >= 0);
		},

		isWarn : function () {
			return (this.level >= 1);
		},

		isInfo : function () {
			return (this.level >= 2);
		},

		isFine : function () {
			return (this.level >= 3);
		},

		isFiner : function () {
			return (this.level >= 4);
		},

		isFinest : function () {
			return (this.level >= 5);
		},

		error : function (dbgString, methodName) {
			if (this.isError()) {
				if (dbgString && dbgString instanceof Error) {
					console.error(this._output("EXCEPTION", methodName, dbgString.toString()));
					this._pushToServer("SEVERE", methodName, dbgString.toString());
					//} else if (dbgString && dbgString.isInstanceOf && dbgString.isInstanceOf(ibm.tivoli.navmodel.Exception)) {
					//    console.error(this._output("EXCEPTION", methodName, dbgString.getMsgText()));
				} else {
					console.error(this._output("ERROR", methodName, dbgString));
					this._pushToServer("SEVERE", methodName, dbgString);
				}
			}
		},

		warn : function (dbgString, methodName) {
			if (this.isWarn()) {
				console.warn(this._output("WARN", methodName, dbgString));
				//this._pushToServer("WARNING", methodName, dbgString); //toggle if we want it
			}
		},

		info : function (dbgString, methodName) {
			if (this.isInfo()) {
				console.info(this._output("INFO", methodName, dbgString));
			}
		},

		fine : function (dbgString, methodName) {
			if (this.isFine()) {
				console.info(this._output("FINE", methodName, dbgString));
			}
		},

		finer : function (dbgString, methodName) {
			if (this.isFiner()) {
				console.info(this._output("FINER", methodName, dbgString));
			}
		},

		finest : function (dbgString, methodName) {
			if (this.isFinest()) {
				console.info(this._output("FINEST", methodName, dbgString));
			}
		},

		entering : function (dbgString, methodName) {
			if (this.isFinest()) {
				console.info(this._output("ENTERING", methodName, dbgString));
			}
		},

		exiting : function (dbgString, methodName) {
			if (this.isFinest()) {
				console.info(this._output("EXITING", methodName, dbgString));
			}
		},

		object : function (dbgObject, methodName) {
			if (this.isFine()) {
				console.log(this._output("OBJECT", methodName, json.stringify(dbgObject)));
			}
		},

		assert : function (predicate, message, methodName) {
			if (this.isFine()) {
				if (predicate === false) {
					console.log(this._output("ASSERT", methodName, "assertion failed: " + message));
				}
			}
		},

		/////////////////////////////
		// private methods
		/////////////////////////////

		_translateLevel : function (className, value) {
			var level = 0;

			switch (value) {
			case "off":
				level = -1;
				break;
			case "error":
				level = 0;
				break;
			case "warn":
				level = 1;
				break;
			case "info":
				level = 2;
				break;
			case "fine":
				level = 3;
				break;
			case "finer":
				level = 4;
				break;
			case "finest":
				level = 5;
				break;
			default:
				console.log("className=" + className + " has invalid level=" + value);
			}

			return level;
		},

		//***** DEBUG::trace=ibm.tivoli.navmodel.Dataset, method=EXITING; event received, type = itemStatusChanged
		//::class.method=com.ibm.usmi.services.updates.tasks.UpdateInstallTask.jobActivationUpdated()
		_output : function (level, method, message) {
			var methodName = method || "(unknown method)", outputMessage = "***** " + level;
			if (message) {
				outputMessage = outputMessage + ": " + message;
			}

			outputMessage = outputMessage + " ::class.method=" + this.className + "." + methodName + "() ::loggername="
					+ this.traceName;
			return outputMessage;
		},

		_pushToServer : function (level, method, message) {
			//nested object here. "logData" is key value used on server
			var parm = {
				logData : {
					severity : level,
					className : this.className,
					method : method,
					message : message
				}
			};
			topic.publish(Constants.TOPIC_LOG_JS_TO_SERVER, parm);
		}
	});
});
